@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: array.c,v 1.223 2009/02/12 21:06:27 srb Exp $
+|| $Id: array.c,v 1.229 2010/07/11 12:39:11 jonasw Exp $
 */
 
 #include "global.h"
@@ -80,7 +80,6 @@ PMOD_EXPORT struct array *real_allocate_array(ptrdiff_t size,
 					      ptrdiff_t extra_space)
 {
   struct array *v;
-  ptrdiff_t e;
 
   if(size+extra_space == 0)
   {
@@ -115,12 +114,12 @@ PMOD_EXPORT struct array *real_allocate_array(ptrdiff_t size,
   v->size = DO_NOT_WARN((INT32)size);
   INIT_PIKE_MEMOBJ(v);
   DOUBLELINK (first_array, v);
-
-  for(e=0;e<v->size;e++)
+  
   {
-    ITEM(v)[e].type=T_INT;
-    ITEM(v)[e].subtype=NUMBER_NUMBER;
-    ITEM(v)[e].u.integer=0;
+    struct svalue *item = ITEM(v);
+    struct svalue *item_end = item + v->size;
+    while (item < item_end)
+      *item++ = svalue_int_zero;
   }
 
   return v;
@@ -994,7 +993,7 @@ static INLINE int lfun_cmp (const struct svalue *a, const struct svalue *b)
   return 2;
 }
 
-INLINE int set_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
+int set_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
   int res, typediff = a->type - b->type;
 
@@ -1090,7 +1089,7 @@ static int switch_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
   }
 }
 
-static int alpha_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
+int alpha_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
   int res, typediff = a->type - b->type;
 
@@ -2391,7 +2390,7 @@ PMOD_EXPORT struct pike_string *implode(struct array *a,
   ret=begin_wide_shared_string(len,max_shift);
   r=MKPCHARP_STR(ret);
   len = del->len;
-  if(e = a->size)
+  if((e = a->size))
     for(ae=a->item;;ae++)
     {
       switch(ae->type)
@@ -2434,19 +2433,22 @@ PMOD_EXPORT struct array *copy_array_recursively(struct array *a,
 #endif
 
   if (!a->size) {
-    add_ref(&empty_array);
-    return array_set_flags(&empty_array, a->flags);
+    ret = (a->flags & ARRAY_WEAK_FLAG) ? &weak_empty_array : &empty_array;
+    add_ref(ret);
+    return ret;
   }
 
   ret=allocate_array_no_init(a->size,0);
-
-  aa.type = T_ARRAY;
-  aa.subtype = 0;
-  aa.u.array = a;
-  bb.type = T_ARRAY;
-  bb.subtype = 0;
-  bb.u.array = ret;
-  low_mapping_insert(m, &aa, &bb, 1);
+  
+  if (m) {
+    aa.type = T_ARRAY;
+    aa.subtype = 0;
+    aa.u.array = a;
+    bb.type = T_ARRAY;
+    bb.subtype = 0;
+    bb.u.array = ret;
+    low_mapping_insert(m, &aa, &bb, 1);
+  }
 
   ret->flags = a->flags & ~ARRAY_LVALUE;
 
@@ -2689,7 +2691,7 @@ static void gc_check_array(struct array *a)
 
 void gc_mark_array_as_referenced(struct array *a)
 {
-  if(gc_mark(a))
+  if(gc_mark(a, T_ARRAY))
     GC_ENTER (a, T_ARRAY) {
       if (a == gc_mark_array_pos)
 	gc_mark_array_pos = a->next;
