@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: sparc.h,v 1.31 2008/05/03 12:33:59 marcus Exp $
+|| $Id: sparc.h,v 1.32 2009/11/21 21:52:28 marcus Exp $
 */
 
 #define PIKE_OPCODE_ALIGN	4
@@ -106,3 +106,11 @@ void sparc_disassemble_code(void *addr, size_t bytes);
     }								\
   } while(0)
 #endif /* !PIKE_DEBUG */
+
+#ifdef __GNUC__
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
+/* Avoid an overoptimization bug in gcc 4.3.4 which caused the address
+ * stored in do_inter_return_label to be at the CALL_MACHINE_CODE line. */
+#define EXIT_MACHINE_CODE() do { __asm__ __volatile__(""); } while(0)
+#endif
+#endif
