@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: gc.h,v 1.136 2008/07/24 20:02:53 mast Exp $
+|| $Id: gc.h,v 1.146 2010/06/22 09:24:32 mast Exp $
 */
 
 #ifndef GC_H
@@ -24,9 +24,10 @@ extern int gc_enabled;
  * total amount of allocated things is this. */
 extern double gc_garbage_ratio_low;
 
-/* When more than this fraction of the cpu time is spent in the gc,
- * aim to minimize it as long as the garbage ratio is less than
- * gc_garbage_ratio_high. */
+/* When more than this fraction of the time is spent in the gc, aim to
+ * minimize it as long as the garbage ratio is less than
+ * gc_garbage_ratio_high. (Note that the intervals currently are
+ * measured in real time since cpu time typically is thread local.) */
 extern double gc_time_ratio;
 
 /* If the garbage ratio gets up to this value, disregard gc_time_ratio
@@ -34,6 +35,12 @@ extern double gc_time_ratio;
  * get any higher. */
 extern double gc_garbage_ratio_high;
 
+/* If there's very little garbage, the gc can settle on extremely long
+ * intervals that won't pick up an increased garbage rate until it's
+ * too late. This is the minimum time ratio between the gc running
+ * time and the gc interval time to avoid that. */
+extern double gc_min_time_ratio;
+
 /* When predicting the next gc interval, use a decaying average with
  * this slowness factor. It should be a value less than 1.0 that
  * specifies the weight to give to the old average value. The
@@ -296,6 +303,7 @@ void describe_location(void *real_memblock,
 		       int depth,
 		       int flags);
 void debug_gc_fatal(void *a, int flags, const char *fmt, ...);
+void debug_gc_fatal_2 (void *a, int type, int flags, const char *fmt, ...);
 void low_describe_something(void *a,
 			    int t,
 			    int indent,
@@ -319,7 +327,7 @@ void debug_really_free_gc_frame(struct gc_frame *l);
 int gc_do_weak_free(void *a);
 void gc_delayed_free(void *a, int type);
 void debug_gc_mark_enqueue(queue_call call, void *data);
-int gc_mark(void *a);
+int real_gc_mark(void *a DO_IF_DEBUG (COMMA int type));
 void gc_move_marker (void *old_thing, void *new_thing);
 PMOD_EXPORT void gc_cycle_enqueue(gc_cycle_check_cb *checkfn, void *data, int weak);
 void gc_cycle_run_queue(void);
@@ -329,6 +337,7 @@ void do_gc_recurse_short_svalue(union anything *u, int type);
 int gc_do_free(void *a);
 size_t do_gc(void *ignored, int explicit_call);
 void f__gc_status(INT32 args);
+void f_implicit_gc_real_time (INT32 args);
 void f_count_memory (INT32 args);
 void cleanup_gc(void);
 
@@ -338,6 +347,12 @@ void cleanup_gc(void);
 #define DMALLOC_TOUCH_MARKER(X, EXPR) (EXPR)
 #endif
 
+#ifdef PIKE_DEBUG
+#define gc_mark(DATA, TYPE) real_gc_mark (DATA, TYPE)
+#else
+#define gc_mark(DATA, TYPE) real_gc_mark (DATA)
+#endif
+
 #define gc_check(VP) \
   DMALLOC_TOUCH_MARKER(VP, real_gc_check(debug_malloc_pass(VP)))
 #define gc_check_weak(VP) \
@@ -362,7 +377,14 @@ void gc_mark_discard_queue();
 #else  /* !GC_MARK_DEBUG */
 
 extern struct pike_queue gc_mark_queue;
-#define gc_mark_enqueue(FN, DATA) enqueue (&gc_mark_queue, (FN), (DATA))
+#define gc_mark_enqueue(FN, DATA) do {					\
+    DO_IF_DEBUG (							\
+      if (Pike_in_gc != GC_PASS_MARK && Pike_in_gc != GC_PASS_ZAP_WEAK)	\
+	gc_fatal ((DATA), 0,						\
+		  "gc_mark_enqueue() called in invalid gc pass.\n");	\
+    );									\
+    enqueue (&gc_mark_queue, (FN), (DATA));				\
+  } while (0)
 #define gc_mark_run_queue() run_queue (&gc_mark_queue)
 #define gc_mark_discard_queue() discard_queue (&gc_mark_queue)
 
@@ -437,6 +459,8 @@ static INLINE int debug_gc_check_weak (void *a, const char *place)
 
 #define gc_fatal \
   fprintf(stderr, "%s:%d: GC fatal:\n", __FILE__, __LINE__), debug_gc_fatal
+#define gc_fatal_2 \
+  fprintf(stderr, "%s:%d: GC fatal:\n", __FILE__, __LINE__), debug_gc_fatal_2
 
 #ifdef PIKE_DEBUG
 
@@ -464,23 +488,35 @@ static INLINE int debug_gc_check_weak (void *a, const char *place)
 
 #define gc_recurse_svalues(S,N)						\
   (Pike_in_gc == GC_PASS_CYCLE ?					\
-   gc_cycle_check_svalues((S), (N)) : gc_mark_svalues((S), (N)))
+   gc_cycle_check_svalues((S), (N)) :					\
+   Pike_in_gc == GC_PASS_MARK || Pike_in_gc == GC_PASS_ZAP_WEAK ?	\
+   gc_mark_svalues((S), (N)) :						\
+   (visit_svalues ((S), (N), REF_TYPE_NORMAL), 0))
 #define gc_recurse_short_svalue(U,T)					\
   (Pike_in_gc == GC_PASS_CYCLE ?					\
-   gc_cycle_check_short_svalue((U), (T)) : gc_mark_short_svalue((U), (T)))
+   gc_cycle_check_short_svalue((U), (T)) :				\
+   Pike_in_gc == GC_PASS_MARK || Pike_in_gc == GC_PASS_ZAP_WEAK ?	\
+   gc_mark_short_svalue((U), (T)) :					\
+   visit_short_svalue ((U), (T), REF_TYPE_NORMAL))
 #define gc_recurse_weak_svalues(S,N)					\
   (Pike_in_gc == GC_PASS_CYCLE ?					\
-   gc_cycle_check_weak_svalues((S), (N)) : gc_mark_weak_svalues((S), (N)))
+   gc_cycle_check_weak_svalues((S), (N)) :				\
+   Pike_in_gc == GC_PASS_MARK || Pike_in_gc == GC_PASS_ZAP_WEAK ?	\
+   gc_mark_weak_svalues((S), (N)) :					\
+   (visit_svalues ((S), (N), REF_TYPE_WEAK), 0))
 #define gc_recurse_weak_short_svalue(U,T)				\
   (Pike_in_gc == GC_PASS_CYCLE ?					\
-   gc_cycle_check_weak_short_svalue((U), (T)) : gc_mark_weak_short_svalue((U), (T)))
+   gc_cycle_check_weak_short_svalue((U), (T)) :				\
+   Pike_in_gc == GC_PASS_MARK || Pike_in_gc == GC_PASS_ZAP_WEAK ?	\
+   gc_mark_weak_short_svalue((U), (T)) :				\
+   visit_short_svalue ((U), (T), REF_TYPE_WEAK))
 
 #define GC_RECURSE_THING(V, T)						\
-  (mc_pass ?								\
-   PIKE_CONCAT3 (visit_,T,_ref) (debug_malloc_pass (V), REF_TYPE_NORMAL) : \
-   (DMALLOC_TOUCH_MARKER(V, Pike_in_gc == GC_PASS_CYCLE) ?		\
-    PIKE_CONCAT(gc_cycle_check_, T)(V, 0) :				\
-    PIKE_CONCAT3(gc_mark_, T, _as_referenced)(V)))
+  (DMALLOC_TOUCH_MARKER(V, Pike_in_gc == GC_PASS_CYCLE) ?		\
+   PIKE_CONCAT(gc_cycle_check_, T)(V, 0) :				\
+   Pike_in_gc == GC_PASS_MARK || Pike_in_gc == GC_PASS_ZAP_WEAK ?	\
+   PIKE_CONCAT3(gc_mark_, T, _as_referenced)(V) :			\
+   PIKE_CONCAT3 (visit_,T,_ref) (debug_malloc_pass (V), REF_TYPE_NORMAL))
 #define gc_recurse_array(V) GC_RECURSE_THING((V), array)
 #define gc_recurse_mapping(V) GC_RECURSE_THING((V), mapping)
 #define gc_recurse_multiset(V) GC_RECURSE_THING((V), multiset)
@@ -624,17 +660,28 @@ PMOD_EXPORT extern visit_ref_cb *visit_ref;
 PMOD_EXPORT extern visit_thing_fn *const visit_fn_from_type[MAX_REF_TYPE + 1];
 PMOD_EXPORT TYPE_T type_from_visit_fn (visit_thing_fn *fn);
 
-static INLINE void real_visit_short_svalue (const union anything *u, TYPE_T t,
-					    int ref_type)
+PMOD_EXPORT TYPE_FIELD real_visit_svalues (struct svalue *s, size_t num,
+					   int ref_type);
+
+static INLINE int real_visit_short_svalue (union anything *u, TYPE_T t,
+					   int ref_type)
 {
   check_short_svalue (u, t);
   if (t <= MAX_REF_TYPE)
     visit_ref (u->ptr, ref_type, visit_fn_from_type[t], NULL);
+  return 0;
 }
 #define visit_short_svalue(U, T, REF_TYPE) \
   (real_visit_short_svalue (debug_malloc_pass ((U)->ptr), (T), (REF_TYPE)))
 
 #ifdef DEBUG_MALLOC
+static INLINE TYPE_FIELD dmalloc_visit_svalues (struct svalue *s, size_t num,
+						int ref_type, char *l)
+{
+  return real_visit_svalues (dmalloc_check_svalues (s, num, l), num, ref_type);
+}
+#define visit_svalues(S, NUM, REF_TYPE)					\
+  dmalloc_visit_svalues ((S), (NUM), (REF_TYPE), DMALLOC_LOCATION())
 static INLINE void dmalloc_visit_svalue (struct svalue *s,
 					 int ref_type, char *l)
 {
@@ -649,6 +696,7 @@ static INLINE void dmalloc_visit_svalue (struct svalue *s,
 #define visit_svalue(S, REF_TYPE) \
   dmalloc_visit_svalue ((S), (REF_TYPE), DMALLOC_LOCATION())
 #else
+#define visit_svalues real_visit_svalues
 static INLINE void visit_svalue (struct svalue *s, int ref_type)
 {
   int t = s->type;
@@ -660,6 +708,7 @@ static INLINE void visit_svalue (struct svalue *s, int ref_type)
 }
 #endif
 
+
 /* Memory counting */
 
 PMOD_EXPORT extern int mc_pass;
