@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: main.c,v 1.236 2009/09/18 16:00:42 grubba Exp $
+|| $Id: main.c,v 1.242 2010/08/26 14:17:24 mast Exp $
 */
 
 #include "global.h"
@@ -43,8 +43,14 @@
 
 #include "pike_embed.h"
 
-#if defined(__linux__) && defined(HAVE_DLOPEN) && defined(HAVE_DLFCN_H)
+#ifdef LIBPIKE
+#if defined(HAVE_DLOPEN) && defined(HAVE_DLFCN_H)
 #include <dlfcn.h>
+#elif !defined(USE_DLL) && defined(USE_MY_WIN32_DLOPEN)
+#include "pike_dlfcn.h"
+#else
+#undef LIBPIKE
+#endif
 #endif
 
 #include "las.h"
@@ -197,8 +203,27 @@ static void set_default_master(const char *bin_name)
 static char libpike_file[MAXPATHLEN * 2];
 static void *libpike;
 
-static void (*init_pike)(const char **argv, const char *file);
-static void (*init_pike_runtime)(void (*exit_cb)(int));
+typedef void (*modfun)(void);
+#ifdef NO_CAST_TO_FUN
+/* Function pointers can't be casted to scalar pointers according to
+ * ISO-C (probably to support true Harward achitecture machines).
+ */
+static modfun CAST_TO_FUN(void *ptr)
+{
+  union {
+    void *ptr;
+    modfun fun;
+  } u;
+  u.ptr = ptr;
+  return u.fun;
+}
+#else /* !NO_CAST_TO_FUN */
+#define CAST_TO_FUN(X)	((modfun)X)
+#endif /* NO_CAST_TO_FUN */
+
+static void (*init_pike_var)(const char **argv, const char *file);
+static void (*init_pike_runtime_var)(void (*exit_cb)(int));
+static void (*add_predefine_var)(char *s);
 #endif /* LIBPIKE */
 
 static void find_lib_dir(int argc, char **argv)
@@ -262,14 +287,14 @@ static void find_lib_dir(int argc, char **argv)
 	 )
 	dir = p+1;
     }
-    if ((p + CONSTANT_STRLEN("pike.so")) >= libpike_file + 2*MAXPATHLEN) {
+    if ((dir + CONSTANT_STRLEN("pike.so")) >= libpike_file + 2*MAXPATHLEN) {
       /* Not likely to happen as long as MASTER_COOKIE is longer than "pike.so".
        */
       fprintf(stderr, "Too long path to pike.so.\n");
       exit(1);
     }
     /* Don't forget the NUL! */
-    memcpy(p, "pike.so", CONSTANT_STRLEN("pike.so") + 1);
+    memcpy(dir, "pike.so", CONSTANT_STRLEN("pike.so") + 1);
   }
 #endif /* LIBPIKE */
 }
@@ -303,26 +328,31 @@ int main(int argc, char **argv)
   }
 #endif /* HAVE_DLINIT */
 
-  if (!(libpike = dlopen(libpike_name, RTLD_NOW))) {
+  if (!(libpike = dlopen(libpike_file, RTLD_NOW))) {
     const char *err = dlerror();
     if (!err) err = "Unknown reason.";
-    fprintf(stderr, "Failed to open %s: %s\n", libpike_name, err);
+    fprintf(stderr, "Failed to open %s: %s\n", libpike_file, err);
     exit(1);
   }
 
 #define LOOKUP(symbol) do {						\
-    if (!(symbol = CAST_TO_FUN(dlsym(libpike, TOSTR(symbol)))) &&	\
-	!(symbol = CAST_TO_FUN(dlsym(libpike, "_" TOSTR(symbol))))) {	\
+    if (!(PIKE_CONCAT(symbol, _var) =					\
+	  CAST_TO_FUN(dlsym(libpike, TOSTR(symbol)))) &&		\
+	!(PIKE_CONCAT(symbol, _var) =					\
+	  CAST_TO_FUN(dlsym(libpike, "_" TOSTR(symbol))))) {		\
       fprintf(stderr, "Missing symbol in %s: " TOSTR(symbol) "\n",	\
-	      libpike_name);						\
+	      libpike_file);						\
       dlclose(libpike);							\
       exit(1);								\
     }									\
   } while(0)
 
   LOOKUP(init_pike);
+#define init_pike init_pike_var
   LOOKUP(init_pike_runtime);
+#define init_pike_runtime init_pike_runtime_var
   LOOKUP(add_predefine);
+#define add_predefine add_predefine_var
   
 #endif /* LIBPIKE */
 
@@ -587,7 +617,7 @@ int main(int argc, char **argv)
 
   if(SETJMP(back))
   {
-    if(throw_severity == THROW_EXIT)
+    if(throw_severity == THROW_EXIT || throw_severity == THROW_THREAD_EXIT)
     {
       num=throw_value.u.integer;
     }else{
