@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: mapping.c,v 1.213 2009/09/22 15:45:09 grubba Exp $
+|| $Id: mapping.c,v 1.218 2010/07/11 19:12:01 mast Exp $
 */
 
 #include "global.h"
@@ -72,7 +72,7 @@ DO_IF_DEBUG(								\
     datasize+=MAPPING_DATA_SIZE(m->data->hashsize, m->data->num_keypairs) / \
       (double) m->data->refs;						\
   }							\
-  size += (INT32) datasize;				\
+  size += (size_t) datasize;				\
 }while(0)
 
 BLOCK_ALLOC_FILL_PAGES(mapping, 2)
@@ -2133,11 +2133,11 @@ PMOD_EXPORT void f_aggregate_mapping(INT32 args)
 PMOD_EXPORT struct mapping *copy_mapping_recursively(struct mapping *m,
 						     struct mapping *p)
 {
+  int not_complex;
   struct mapping *ret;
   INT32 e;
   struct keypair *k;
   struct mapping_data  *md;
-  struct svalue aa, bb;
 
 #ifdef PIKE_DEBUG
   if(m->data->refs <=0)
@@ -2148,18 +2148,26 @@ PMOD_EXPORT struct mapping *copy_mapping_recursively(struct mapping *m,
   if(d_flag > 1) check_mapping_type_fields(m);
 #endif
 
-  if(!((m->data->val_types | m->data->ind_types) & BIT_COMPLEX))
-    return copy_mapping(m);
+  if((m->data->val_types | m->data->ind_types) & BIT_COMPLEX) {
+    not_complex = 0;
+    ret=allocate_mapping(MAP_SLOTS(m->data->size));
+  }
+  else {
+    not_complex = 1;
+    ret = copy_mapping(m);
+  }
 
-  ret=allocate_mapping(MAP_SLOTS(m->data->size));
+  if (p) {
+    struct svalue aa, bb;
+    aa.type = T_MAPPING;
+    aa.u.mapping = m;
+    bb.type = T_MAPPING;
+    bb.u.mapping = ret;
+    mapping_insert(p, &aa, &bb);
+  }
 
-  aa.type = T_MAPPING;
-  aa.subtype = 0;
-  aa.u.mapping = m;
-  bb.type = T_MAPPING;
-  bb.subtype = 0;
-  bb.u.mapping = ret;
-  mapping_insert(p, &aa, &bb);
+  if (not_complex)
+    return ret;
 
   ret->data->flags = m->data->flags;
 
@@ -2296,7 +2304,7 @@ void check_mapping(const struct mapping *m)
   {
     if(Pike_in_gc)
     {
-      fprintf(stderr,"Pike was in GC stage %d when this fatal occured:\n",Pike_in_gc);
+      fprintf(stderr,"Pike was in GC stage %d when this fatal occurred:\n",Pike_in_gc);
       Pike_in_gc=0;
     }
     
@@ -2583,7 +2591,7 @@ void gc_mark_mapping_as_referenced(struct mapping *m)
   debug_malloc_touch(m);
   debug_malloc_touch(m->data);
 
-  if(gc_mark(m))
+  if(gc_mark(m, T_MAPPING))
     GC_ENTER (m, T_MAPPING) {
       struct mapping_data *md = m->data;
 
@@ -2596,7 +2604,8 @@ void gc_mark_mapping_as_referenced(struct mapping *m)
 	DOUBLELINK(first_mapping, m); /* Linked in first. */
       }
 
-      if(gc_mark(md) && ((md->ind_types | md->val_types) & BIT_COMPLEX)) {
+      if(gc_mark(md, T_MAPPING_DATA) &&
+	 ((md->ind_types | md->val_types) & BIT_COMPLEX)) {
 	TYPE_FIELD ind_types = 0, val_types = 0;
 	if (MAPPING_DATA_IN_USE(md)) {
 	  /* Must leave the mapping data untouched if it's busy. */
