@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: acconfig.h,v 1.6 2003/05/19 19:22:54 mast Exp $
+|| $Id: acconfig.h,v 1.9 2009/06/28 10:08:26 grubba Exp $
 */
 
 #ifndef GMP_MACHINE_H
@@ -41,6 +41,9 @@
 /* Define this if you have mpz_fits_ulong_p */
 #undef HAVE_MPZ_FITS_ULONG_P
 
+/* Define this if you have mpz_bin_ui */
+#undef HAVE_MPZ_BIN_UI
+
 /* Define if your cpp supports the ANSI concatenation operator ## */
 #undef HAVE_ANSI_CONCAT
 
@@ -56,4 +59,13 @@
 /* Define if mpz_set_si works for LONG_MIN. */
 #undef MPZ_SET_SI_WORKS
 
+/* Define if GMP_LIMB_BITS lies. */
+#undef PIKE_GMP_LIMB_BITS_INVALID
+
+/* Define to the actual number of numb bits if known. */
+#undef PIKE_GMP_NUMB_BITS
+
+/* Defined if mpz_t has _mp_alloc in it. */
+#undef MPZ_T_HAS__MP_ALLOC
+
 #endif
