@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: mpz_glue.c,v 1.184 2009/06/28 10:08:26 grubba Exp $
+|| $Id: mpz_glue.c,v 1.190 2010/05/29 14:00:31 mast Exp $
 */
 
 #include "global.h"
@@ -618,12 +618,59 @@ static void mpzmod_get_int(INT32 args)
 #endif /* AUTO_BIGNUM */
 }
 
+static INT32 crc_table[256];
+
+static void init_crc_table(void)
+{
+  int i;
+  for (i = 0; i < 256; i++) {
+    int j;
+    INT32 crc = i << 24;
+    for (j = 0; j < 8; j++) {
+      if (crc < 0) {
+	crc = (crc << 1)^0x04c11db7L;
+      } else {
+	crc <<= 1;
+      }
+    }
+    crc_table[i] = crc;
+  }
+}
+
 /*! @decl int __hash()
+ *!
+ *!   Calculate a hash of the value.
+ *!
+ *! @note
+ *!   Prior to Pike 7.8.359 this function returned the low
+ *!   32-bits as an unsigned integer. This could in some
+ *!   common cases lead to very unbalanced mappings.
+ *!
+ *! @seealso
+ *!   @[hash_value()]
  */
 static void mpzmod___hash(INT32 args)
 {
+  MP_INT *mpz = THIS;
+
+  /* Calculate the CRC32 of the limbs.
+   * NOTE: LSB first!
+   */
+  INT32 crc = 0;
+  size_t pos;
+  for (pos = 0; pos < mpz_size(mpz); pos++) {
+    mp_limb_t x = MPZ_GETLIMBN (mpz, pos);
+    size_t i;
+    for (i=0; i<sizeof(mp_limb_t); i++) {
+      crc = (crc >> 8) ^ crc_table[((crc >> 24) ^ x) & 0xff];
+      x >>= 8;
+    }
+  }
   pop_n_elems(args);
-  push_int(mpz_get_si(THIS));
+  if (mpz_sgn(mpz) < 0)
+    push_int(-crc);
+  else
+    push_int(crc);
 }
 
 /*! @decl float cast_to_float()
@@ -706,6 +753,7 @@ struct pike_string *low_get_mpz_digits(MP_INT *mpz, int base)
  */
 static void mpzmod_get_string(INT32 args)
 {
+  /* Also called as json_encode (with some arguments). */
   pop_n_elems(args);
   push_string(low_get_mpz_digits(THIS, 10));
 }
@@ -2088,7 +2136,7 @@ static void mpzmod_popcount(INT32 args)
        Pike_sp[-1].u.integer=-1;
 #endif
     break;
-  default:
+  default:;
 #ifdef PIKE_DEBUG
     Pike_fatal("Gmp.mpz->popcount: Unexpected sign!\n");
 #endif
@@ -2121,7 +2169,30 @@ static void mpzmod_random(INT32 args)
   dmalloc_touch_svalue(Pike_sp);
   PUSH_REDUCED(res);
 }
-  
+
+/*! @decl string(0..255) _encode()
+ */  
+static void mpzmod__encode(INT32 args)
+{
+  pop_n_elems(args);
+  /* 256 would be better, but then negative numbers
+   * won't work... /Hubbe
+   */
+  push_int(36);
+  mpzmod_digits(1);
+}
+
+/*! @decl void _decode(string(0..255))
+ */
+static void mpzmod__decode(INT32 args)
+{
+  /* 256 would be better, but then negative numbers
+   * won't work... /Hubbe
+   */
+  push_int(36);
+  mpzmod_create(args+1);
+}
+
 /*! @endclass
  */
 
@@ -2300,6 +2371,8 @@ static void pike_mp_free (void *ptr, size_t size)
                ID_PROTECTED);                                           \
   									\
   ADD_FUNCTION("digits", mpzmod_digits,tFunc(tOr(tVoid,tInt),tStr), 0);	\
+  ADD_FUNCTION("encode_json", mpzmod_get_string,			\
+	       tFunc(tOr(tVoid,tInt) tOr(tVoid,tInt),tStr), 0);		\
   ADD_FUNCTION("_sprintf", mpzmod__sprintf, tFunc(tInt tMapping,tStr),  \
                ID_PROTECTED);                                           \
   ADD_FUNCTION("size", mpzmod_size,tFunc(tOr(tVoid,tInt),tInt), 0);	\
@@ -2335,6 +2408,10 @@ static void pike_mp_free (void *ptr, size_t size)
 									\
   ADD_FUNCTION("_random",mpzmod_random,tFunc(tNone,tMpz_ret),0);	\
   									\
+  ADD_FUNCTION("_encode",mpzmod__encode,tFunc(tNone,tStr8),0);		\
+  									\
+  ADD_FUNCTION("_decode",mpzmod__decode,tFunc(tStr8,tVoid),0);		\
+  									\
   set_init_callback(init_mpz_glue);					\
   set_exit_callback(exit_mpz_glue);					\
   set_gc_recurse_callback (gc_recurse_mpz);
@@ -2349,6 +2426,8 @@ PIKE_MODULE_INIT
   }
 #endif /* PIKE_DEBUG */
 
+  init_crc_table();
+
   /* Make sure that gmp uses the same malloc functions as we do since
    * we got code that frees blocks allocated inside gmp (e.g.
    * mpf.get_string). This also ensures that gmp uses dlmalloc if we
