@@ -2,24 +2,9 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: any.c,v 1.34 2006/09/12 09:47:33 grubba Exp $
+|| $Id: any.c,v 1.35 2009/12/09 14:29:58 tor Exp $
 */
 
-/*
-**! module Image
-**! submodule ANY
-**!
-**!	This method calls the other decoding methods
-**!	and has some heuristics for what type of image
-**!	this is.
-**!
-**!	Methods:
-**!	<ref>decode</ref>, <ref>decode_alpha</ref>,
-**!	<ref>_decode</ref>
-**!
-**! see also: Image
-**!
-*/
 #include "global.h"
 
 #include <math.h>
@@ -37,6 +22,17 @@
 #include "encodings.h"
 
 
+/*! @module Image
+ */
+
+/*! @module ANY
+ *!
+ *!  These method calls other decoding methods
+ *!  and has some heuristics for what type of image
+ *!  this is.
+ *!
+ */
+
 #define sp Pike_sp
 
 /* PNG module uses "type" for something else than what we want to use
@@ -58,29 +54,32 @@ static void fix_png_mapping(void)
 }
 
 
-/*
-**! method mapping _decode(string data)
-**! method object decode(string data)
-**! method object decode_alpha(string data)
-**!	Tries heuristics to find the correct method 
-**!	of decoding the data, then calls that method.
-**!
-**! 	The result of _decode() is a mapping that contains
-**!	<pre>
-**!		"type":image data type (ie, "image/jpeg" or similar)
-**!		"image":the image object,
-**!		"alpha":the alpha channel or 0 if N/A
-**!	</pre>
-**!
-**! note
-**!	Throws upon failure.
-*/
-
+/*! @decl mapping _decode(string data)
+ *! @decl object decode(string data)
+ *! @decl object decode_alpha(string data)
+ *!
+ *! Tries heuristics to find the correct method
+ *! of decoding the data, then calls that method.
+ *!
+ *! The result of _decode() is a mapping that contains
+ *!
+ *! @mapping
+ *!   @member string "type"
+ *!     File type information as MIME type (ie "image/jpeg" or similar)
+ *!   @member Image.Image "image"
+ *!     the image object
+ *!   @member int alpha
+ *!     the alpha channel or 0 if N/A
+ *! @endmapping
+ *!
+ *! @note
+ *!	Throws upon failure.
+ */
 void image_any__decode(INT32 args)
 {
    if (args!=1 || sp[-args].type!=T_STRING)
       Pike_error("Image.ANY.decode: illegal arguments\n");
-   
+
    if (sp[-args].u.string->len<4)
       Pike_error("Image.ANY.decode: too short string\n");
 
@@ -191,7 +190,7 @@ void image_any__decode(INT32 args)
 	       image_xwd__decode(1);
 	       return; /* done */
 	 }
-	 
+
 	 goto unknown_format;
 
       default:
@@ -202,7 +201,7 @@ void image_any__decode(INT32 args)
 	  goto simple_image;
 	}
 unknown_format:
-	 Pike_error("Unknown image format.\n");	 
+	 Pike_error("Unknown image format.\n");
    }
 
 simple_image:
@@ -216,11 +215,34 @@ simple_image:
    return;
 }
 
+
+
+/*! @decl mapping decode_header(string data)
+ *!
+ *! Tries heuristics to find the correct method
+ *! of decoding the header, then calls that method.
+ *!
+ *! The resulting mapping depends on wich decode_header method that
+ *! is executed, but these keys will probably exist
+ *!
+ *! @mapping
+ *!   @member int "xsize"
+ *!   @member int "ysize"
+ *!       Size of image
+ *!   @member string "type"
+ *!     File type information as MIME type.
+ *!   @member string "color_space"
+ *!     Color space of image.
+ *! @endmapping
+ *!
+ *! @note
+ *!	Throws upon failure.
+ */
 void image_any_decode_header(INT32 args)
 {
    if (args!=1 || sp[-args].type!=T_STRING)
       Pike_error("Image.ANY.decode_header: illegal arguments\n");
-   
+
    if (sp[-args].u.string->len<4)
       Pike_error("Image.ANY.decode_header: too short string\n");
 
@@ -273,6 +295,15 @@ void image_any_decode_header(INT32 args)
       case CHAR2('F','O'):
 	 Pike_error("Image.ANY.decode: decoding of ILBM header unimplemented\n");
 
+      case CHAR2('I','I'):	/* Little endian. */
+      case CHAR2('M','M'):	/* Big endian. */
+	/* TIFF */
+	push_text("Image.TIFF.decode_header");
+	SAFE_APPLY_MASTER("resolv_or_error",1);
+	stack_swap();
+	f_call_function(2);
+	return;
+
       case CHAR2('B','M'):
 	 /* BMP */
 	 img_bmp_decode_header(1);
@@ -293,6 +324,15 @@ void image_any_decode_header(INT32 args)
 	 image_tim_f_decode_header(1);
 	 return;
 
+      case CHAR2(0xc5, 0xd0):
+      case CHAR2('%','!'):
+	/* PS */
+	push_text("Image.PS.decode_header");
+	SAFE_APPLY_MASTER("resolv_or_error",1);
+	stack_swap();
+	f_call_function(2);
+	break;
+
       case CHAR2(0,0):
 	 switch (CHAR2(sp[-args].u.string->str[2],sp[-args].u.string->str[3]))
 	 {
@@ -301,12 +341,12 @@ void image_any_decode_header(INT32 args)
 	       image_xwd_decode_header(1);
 	       return; /* done */
 	 }
-	 
+
 	 goto unknown_format;
 
       default:
 unknown_format:
-	 Pike_error("Unknown image format.\n");	 
+	 Pike_error("Unknown image format.\n");
    }
 }
 
@@ -324,6 +364,11 @@ void image_any_decode_alpha(INT32 args)
    f_index(2);
 }
 
+/*! @endmodule
+ */
+
+/*! @endmodule
+ */
 
 /** module *******************************************/
 
