@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: png.c,v 1.98 2009/08/10 16:10:00 nilsson Exp $
+|| $Id: png.c,v 1.101 2010/05/29 16:12:12 nilsson Exp $
 */
 
 #include "global.h"
@@ -1428,6 +1428,10 @@ static void img_png_decode(INT32 args, int mode)
 
          default:
             if(mode==MODE_IMAGE_ONLY) break;
+
+            /* Private chunks from Adobe Fireworks: prVW, mkBF, mkBS,
+               mkBT, mkTS */
+
 	    ref_push_string(b->item[1].u.string);
 	    ref_push_string(b->item[0].u.string);
 	    mapping_insert(m,sp-1,sp-2);
@@ -1466,9 +1470,12 @@ static void img_png_decode(INT32 args, int mode)
      pop_stack();
 
      if(trns)
-       CALL_AND_UNSET_ONERROR(err);
+       UNSET_ONERROR(err);
    }
 
+   if (trns)
+     png_free_string (trns);
+
    if ( mode != MODE_IMAGE_ONLY )
    {
      push_int(ihdr.type);
@@ -1705,9 +1712,24 @@ static void image_png_encode(INT32 args)
 		  else bit-=bpp;
 		  ts++;
 	       }
+	       if (bit != (8 - bpp)) {
+		 /* The last byte wasn't filled fully,
+		  * so we need to pad with a few bits.
+		  */
+		 d++;
+	       }
 	    }
 	 }
 
+#ifdef PIKE_DEBUG
+	 if (d != (unsigned char *)(ps->str + ps->len)) {
+	   Pike_fatal("PNG data doesn't align properly "
+		      "%d x %d (%d bpp) len: %ld, got: %ld.\n",
+		      img->xsize, img->ysize, bpp,
+		      (long)ps->len, (long)(d - (unsigned char *)ps->str));
+	 }
+#endif
+
          push_string(end_shared_string(ps));
 	 free(tmp);
       }
