@@ -2,14 +2,17 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: polyfill.c,v 1.46 2006/03/07 09:06:37 peter Exp $
+|| $Id: polyfill.c,v 1.52 2007/05/04 18:31:35 grubba Exp $
 */
 
 #include "global.h"
-RCSID("$Id: polyfill.c,v 1.46 2006/03/07 09:06:37 peter Exp $");
 
+#if 0
 /* Prototypes are needed for these */
+/* Should really exist in math.h. This prototype might conflict with
+ * dllimport stuff on Windows. /mast */
 extern double floor(double);
+#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -85,7 +88,7 @@ struct vertex
 {
    double x, y;
    struct vertex *next;    /* total list, sorted downwards */
-   struct line_list *below,*above; /* childs */
+   struct line_list *below,*above; /* children */
    int done;
 };
 
@@ -136,6 +139,7 @@ static void vertex_connect(struct vertex *above,
    above->below = c;
 
    d = malloc(sizeof(struct line_list));
+   if(!d) { free(c); return; }
    d->above = above; d->below = below;
    d->next = below->above;
    d->dx = c->dx;
@@ -290,6 +294,7 @@ static void add_vertices(struct line_list **first,
 
 
       c=malloc(sizeof(struct line_list));
+      if(!c) return;
       *c=*what;
       c->next=*ins;
       *ins=c;
@@ -725,7 +730,7 @@ static INLINE struct vertex *polyfill_begin(void)
    return NULL;
 }
 
-static INLINE struct vertex *polyfill_add(struct vertex *top,
+static INLINE struct vertex *polyfill_add(struct vertex **top,
 					  struct array *a,
 					  int arg,
 					  char* what)
@@ -733,20 +738,18 @@ static INLINE struct vertex *polyfill_add(struct vertex *top,
    struct vertex *first,*last,*cur = NULL;
    int n;
 
-   for (n=0; n<a->size; n++)
-      if (a->item[n].type!=T_FLOAT &&
-	  a->item[n].type!=T_INT)
-      {
-	 polyfill_free(top);
-	 Pike_error("Illegal argument %d to %s, array index %d is not int nor float\n",arg,what,n);
-	 return NULL;
-      }
+   if( (a->type_field & ~(BIT_INT|BIT_FLOAT)) &&
+       (array_fix_type_field(a) & ~(BIT_INT|BIT_FLOAT)) ) {
+     polyfill_free(*top);
+     Pike_error("Illegal argument %d to %s. %d Expected array(float|int).\n",arg,what, a->type_field);
+     return NULL;
+   }
 
    if (a->size<6) 
    {
-      return top; 
+      return *top; 
 #if 0
-      polyfill_free(top);
+      polyfill_free(*top);
       Pike_error("Illegal argument %d to %s, too few vertices (min 3)\n", arg, what);
       return NULL; /* no polygon with less then tree corners */
 #endif
@@ -756,16 +759,20 @@ static INLINE struct vertex *polyfill_add(struct vertex *top,
 
    last = first = vertex_new(DO_NOT_WARN(POINT(a,0)),
 			     DO_NOT_WARN(POINT(a,1)),
-			     &top);
+			     top);
 
-   if (!last) return NULL;
+   if (!last) {
+      return NULL;
+   }
 
    for (n=2; n+1<a->size; n+=2)
    {
       cur = vertex_new(DO_NOT_WARN(POINT(a,n)),
 		       DO_NOT_WARN(POINT(a,n+1)),
-		       &top);
-      if (!cur) return NULL;
+		       top);
+      if (!cur) {
+	 return NULL;
+      }
       if (cur->y<last->y)
 	 vertex_connect(cur,last);
       else if (cur->y>last->y)
@@ -787,20 +794,20 @@ static INLINE struct vertex *polyfill_add(struct vertex *top,
    else
       vertex_connect(first,cur);
 
-   return top;
+   return *top;
 }
 
 void image_polyfill(INT32 args)
 {
    struct vertex *v;
    double *buf;
+   ONERROR err;
 
    if (!THIS->img)
       Pike_error("Image.Image->polyfill: no image\n");
 
-   buf=malloc(sizeof(double)*(THIS->xsize+1));
-   if (!buf)
-      Pike_error("Image.Image->polyfill: out of memory\n");
+   buf=xalloc(sizeof(double)*(THIS->xsize+1));
+   SET_ONERROR(err, free, buf);
 
    v=polyfill_begin();
 
@@ -811,10 +818,11 @@ void image_polyfill(INT32 args)
       if (sp[-1].type!=T_ARRAY)
       {
 	 polyfill_free(v);
-	 Pike_error("Image.Image->polyfill: Illegal argument %d, expected array\n",
-	       args);
+	 SIMPLE_BAD_ARG_ERROR("Image.Image->polyfill", args,
+			      "array(int|float)");
       }
-      if ((v_tmp=polyfill_add(v, sp[-1].u.array, args, "Image.Image->polyfill()"))) {
+      if ((v_tmp=polyfill_add(&v, sp[-1].u.array, args,
+			      "Image.Image->polyfill()"))) {
 	 v = v_tmp;
       } else {
 	 polyfill_free(v);
@@ -832,7 +840,8 @@ void image_polyfill(INT32 args)
    polyfill_some(THIS,v,buf);
    
    polyfill_free(v);
-   
+
+   UNSET_ONERROR(err);
    free(buf);
 
    ref_push_object(THISOBJ);
