@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: mysql.c,v 1.117 2009/07/24 12:42:39 mast Exp $
+|| $Id: 44d8150458f480f9d5b37fd16d16c3dfa737b7b2 $
 */
 
 /*
@@ -157,6 +157,9 @@ static MUTEX_T stupid_port_lock;
 #define MYSQL_DISALLOW()
 #endif /* _REENTRANT */
 
+#define PIKE_MYSQL_FLAG_STORE_RESULT	1
+#define PIKE_MYSQL_FLAG_TYPED_RESULT	2
+
 #define CHECK_8BIT_NONBINARY_STRING(FUNC, ARG) do {			\
     if (sp[ARG-1-args].type != T_STRING ||				\
 	sp[ARG-1-args].u.string->size_shift ||				\
@@ -214,12 +217,10 @@ static void exit_mysql_struct(struct object *o)
     free_mapping (PIKE_MYSQL->options);
     PIKE_MYSQL->options = NULL;
   }
-#ifndef HAVE_MYSQL_SET_CHARACTER_SET
   if (PIKE_MYSQL->conn_charset) {
     free_string (PIKE_MYSQL->conn_charset);
     PIKE_MYSQL->conn_charset = NULL;
   }
-#endif
 
   MYSQL_ALLOW();
 
@@ -347,14 +348,48 @@ static void pike_mysql_set_options(struct mapping *options)
 		  val->u.string->str);
   }
 #endif /* MYSQL_SET_CHARSET_DIR */
-#ifdef HAVE_MYSQL_SET_CHARSET_NAME
-  if ((val = simple_mapping_string_lookup(options, "mysql_charset_name")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift)) {
-    mysql_options(PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_NAME,
-		  val->u.string->str);
-  }
-#endif /* MYSQL_SET_CHARSET_NAME */
 #endif /* HAVE_MYSQL_OPTIONS */
+
+  if ((val = simple_mapping_string_lookup(options, "mysql_charset_name")) &&
+      (val->type == T_STRING) && (!val->u.string->size_shift) &&
+      !string_has_null (val->u.string)) {
+    if (PIKE_MYSQL->conn_charset)
+      free (PIKE_MYSQL->conn_charset);
+    copy_shared_string (PIKE_MYSQL->conn_charset, val->u.string);
+  }
+}
+
+static void low_query(INT32 args, char *name, int flags);
+
+static void connection_set_charset()
+{
+#ifdef HAVE_MYSQL_SET_CHARACTER_SET
+  int res;
+  MYSQL *mysql = PIKE_MYSQL->mysql;
+  struct pike_string *charset = PIKE_MYSQL->conn_charset;
+  MYSQL_ALLOW();
+  res = mysql_set_character_set (mysql, charset->str);
+  MYSQL_DISALLOW();
+  if (res) {
+    const char *err;
+    MYSQL_ALLOW();
+    err = mysql_error(mysql);
+    MYSQL_DISALLOW();
+    Pike_error("Setting the charset failed: %s\n", err);
+  }
+
+#else
+  /* Old libs (< 4.1.13) doesn't support changing the connection
+   * charset. We emulate it by setting the charset ourselves. Note
+   * that this doesn't work with mysql_real_escape_string, but that
+   * function isn't used. */
+  push_constant_text ("SET NAMES '");
+  ref_push_string (PIKE_MYSQL->conn_charset);
+  push_constant_text ("'");
+  f_add (3);
+  low_query (1, "set_charset", PIKE_MYSQL_FLAG_STORE_RESULT);
+  pop_stack();
+#endif
 }
 
 static void pike_mysql_reconnect (int reconnect)
@@ -404,10 +439,11 @@ static void pike_mysql_reconnect (int reconnect)
     options = (unsigned int)val->u.integer;
   }
 
-#if !defined (HAVE_MYSQL_SET_CHARACTER_SET) && defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
+#if defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
   if (PIKE_MYSQL->conn_charset)
     mysql_options (mysql, MYSQL_SET_CHARSET_NAME,
 		   PIKE_MYSQL->conn_charset->str);
+#define RECONNECT_CHARSET_IS_SET
 #endif
 
   MYSQL_ALLOW();
@@ -488,6 +524,11 @@ static void pike_mysql_reconnect (int reconnect)
       }
     }
   }
+
+#ifndef RECONNECT_CHARSET_IS_SET
+  if (PIKE_MYSQL->conn_charset)
+    connection_set_charset();
+#endif
 }
 
 /*
@@ -642,24 +683,13 @@ static void f_create(INT32 args)
 
   pike_mysql_reconnect (0);
 
-#ifndef HAVE_MYSQL_SET_CHARACTER_SET
 #ifdef HAVE_MYSQL_CHARACTER_SET_NAME
-  {
+  if (!PIKE_MYSQL->conn_charset) {
     const char *charset = mysql_character_set_name (PIKE_MYSQL->mysql);
-    if (PIKE_MYSQL->conn_charset)
-      free_string (PIKE_MYSQL->conn_charset);
-    if (charset)
+    if (charset) /* Just paranoia; mysql_character_set_name should
+		  * always return a string. */
       PIKE_MYSQL->conn_charset = make_shared_string (charset);
-    else
-      /* Just paranoia; mysql_character_set_name should always return
-       * a string. */
-      PIKE_MYSQL->conn_charset = NULL;
   }
-#else
-  if (PIKE_MYSQL->conn_charset)
-    free_string (PIKE_MYSQL->conn_charset);
-  PIKE_MYSQL->conn_charset = NULL;
-#endif
 #endif
 
   pop_n_elems(args);
@@ -686,9 +716,12 @@ static void mysql__sprintf(INT32 args)
 
       if (mysql) {
 	const char *info;
-	MYSQL_ALLOW();
+	/* _sprintf functions must not hang. mysql_get_host_info is
+	 * safe to execute anyway, because it only returns a field in
+	 * the MYSQL struct. */
+	/* MYSQL_ALLOW(); */
 	info = mysql_get_host_info(mysql);
-	MYSQL_DISALLOW();
+	/* MYSQL_DISALLOW(); */
 	push_text("mysql(/*%s%s*/)");
 	push_text(info ? info : "");
 #ifdef HAVE_MYSQL_SSL
@@ -858,8 +891,6 @@ static void f_select_db(INT32 args)
   pop_n_elems(args);
 }
 
-#define PIKE_MYSQL_FLAG_STORE_RESULT	1
-
 static void low_query(INT32 args, char *name, int flags)
 {
   MYSQL *mysql = PIKE_MYSQL->mysql;
@@ -880,6 +911,12 @@ static void low_query(INT32 args, char *name, int flags)
   query = sp[-args].u.string->str;
   qlen = sp[-args].u.string->len;
 
+  /* NB: mysql_real_query() and mysql_query() call mysql_read_query_result()
+   *     which may have inlined a call to send_file_to_server(), which
+   *     allocates a buffer of IO_SIZE*15 (ie 0xf000) bytes on the stack.
+   */
+  check_c_stack(0x10000);
+
   if (mysql) {
     MYSQL_ALLOW();
 
@@ -901,15 +938,17 @@ static void low_query(INT32 args, char *name, int flags)
   }
   if (mysql && tmp) {
     /* Check if we need to reconnect. */
-#if defined(CR_SERVER_GONE_ERROR) && defined(CR_UNKNOWN_ERROR)
+#if defined(CR_SERVER_GONE_ERROR) && defined(CR_SERVER_LOST) && \
+  defined(CR_UNKNOWN_ERROR)
     int eno = mysql_errno(mysql);
     if ((eno == CR_SERVER_GONE_ERROR) ||
+	(eno == CR_SERVER_LOST) ||
 	(eno == CR_UNKNOWN_ERROR)) {
       mysql = NULL;
     }
-#else /* !CR_SERVER_GONE_ERROR || !CR_UNKNOWN_ERROR */
+#else /* !CR_SERVER_GONE_ERROR || !CR_SERVER_LOST || !CR_UNKNOWN_ERROR */
     mysql = NULL;
-#endif /* CR_SERVER_GONE_ERROR && CR_UNKNOWN_ERROR */
+#endif /* CR_SERVER_GONE_ERROR && CR_SERVER_LOST && CR_UNKNOWN_ERROR */
   }
   if (!mysql) {
     /* The connection might have been closed. */
@@ -988,7 +1027,12 @@ static void low_query(INT32 args, char *name, int flags)
 
     /* Create the object */
     ref_push_object(Pike_fp->current_object);
-    push_object(o = clone_object(mysql_result_program, 1));
+    if (flags & PIKE_MYSQL_FLAG_TYPED_RESULT) {
+      push_int(1);
+      push_object(o = clone_object(mysql_result_program, 2));
+    } else {
+      push_object(o = clone_object(mysql_result_program, 1));
+    }
 
     /* Set the result. */
     if ((!(res = (struct precompiled_mysql_result *)
@@ -1010,6 +1054,17 @@ static void f_streaming_query(INT32 args)
   low_query(args, "streaming_query", 0);
 }
 
+static void f_big_typed_query(INT32 args)
+{
+  low_query(args, "big_typed_query",
+	    PIKE_MYSQL_FLAG_STORE_RESULT | PIKE_MYSQL_FLAG_TYPED_RESULT);
+}
+
+static void f_streaming_typed_query(INT32 args)
+{
+  low_query(args, "streaming_typed_query", PIKE_MYSQL_FLAG_TYPED_RESULT);
+}
+
 
 /*! @decl void create_db(string database)
  *!
@@ -1751,58 +1806,20 @@ static void f_set_charset (INT32 args)
     SIMPLE_ARG_ERROR ("set_charset", 0,
 		      "The charset name cannot contain a NUL character.");
 
-#ifdef HAVE_MYSQL_SET_CHARACTER_SET
-  {
-    int res;
-    MYSQL *mysql = PIKE_MYSQL->mysql;
-    MYSQL_ALLOW();
-    res = mysql_set_character_set (mysql, charset->str);
-    MYSQL_DISALLOW();
-    if (res) {
-      const char *err;
-      MYSQL_ALLOW();
-      err = mysql_error(mysql);
-      MYSQL_DISALLOW();
-      Pike_error("Setting the charset failed: %s\n", err);
-    }
-  }
-
-#else  /* !HAVE_MYSQL_SET_CHARACTER_SET */
-  push_constant_text ("SET NAMES '");
-  ref_push_string (charset);
-  push_constant_text ("'");
-  f_add (3);
-  low_query (1, "set_charset", PIKE_MYSQL_FLAG_STORE_RESULT);
-  args++;
   if (PIKE_MYSQL->conn_charset)
     free_string (PIKE_MYSQL->conn_charset);
   copy_shared_string (PIKE_MYSQL->conn_charset, charset);
-#endif
-
+  connection_set_charset();
   pop_n_elems (args);
 }
 
 static void f_get_charset (INT32 args)
 {
   pop_n_elems (args);
-#ifdef HAVE_MYSQL_SET_CHARACTER_SET
-  {
-    /* mysql_character_set_name should always exist if
-     * mysql_set_character_set exists. */
-    const char *charset = mysql_character_set_name (PIKE_MYSQL->mysql);
-    if (charset)
-      push_text (charset);
-    else
-      /* Just paranoia; mysql_character_set_name should always return
-       * a string. */
-      push_constant_text ("latin1");
-  }
-#else
   if (PIKE_MYSQL->conn_charset)
     ref_push_string (PIKE_MYSQL->conn_charset);
   else
     push_constant_text ("latin1");
-#endif
 }
 
 static void f__can_send_as_latin1 (INT32 args)
@@ -1881,6 +1898,11 @@ PIKE_MODULE_INIT
   /* function(string:int|object) */
   ADD_FUNCTION("streaming_query",
 	       f_streaming_query,tFunc(tStr,tObj), ID_PUBLIC);
+  ADD_FUNCTION("big_typed_query",
+	       f_big_typed_query,tFunc(tStr,tObj), ID_PUBLIC);
+  /* function(string:int|object) */
+  ADD_FUNCTION("streaming_typed_query",
+	       f_streaming_typed_query,tFunc(tStr,tObj), ID_PUBLIC);
 #ifdef USE_OLD_FUNCTIONS
   /* function(string:void) */
   ADD_FUNCTION("create_db", f_create_db,tFunc(tStr,tVoid), ID_PUBLIC);
@@ -1921,7 +1943,7 @@ PIKE_MODULE_INIT
   add_integer_constant( "CLIENT_COMPRESS", CLIENT_COMPRESS, 0);
   add_integer_constant( "CLIENT_FOUND_ROWS", CLIENT_FOUND_ROWS, 0);
   add_integer_constant( "CLIENT_IGNORE_SPACE", CLIENT_IGNORE_SPACE, 0);
-#ifdef HAVE_CLIENT_INTERACTIVE
+#ifdef CLIENT_INTERACTIVE
   add_integer_constant( "CLIENT_INTERACTIVE", CLIENT_INTERACTIVE, 0);
 #endif
   add_integer_constant( "CLIENT_NO_SCHEMA", CLIENT_NO_SCHEMA, 0);
