@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: result.c,v 1.44 2009/09/09 15:48:04 grubba Exp $
+|| $Id: 88942e87ffdbce96fea4c90712164b565236bef1 $
 */
 
 /*
@@ -60,6 +60,7 @@
 #include "pike_error.h"
 #include "builtin_functions.h"
 #include "las.h"
+#include "port.h"
 #include "threads.h"
 #include "multiset.h"
 #include "bignum.h"
@@ -94,6 +95,15 @@
 #ifndef BINARY_FLAG
 #define BINARY_FLAG	128
 #endif
+#ifndef FIELD_TYPE_BIT
+#define FIELD_TYPE_BIT 16
+#endif
+#ifndef FIELD_TYPE_NEWDECIMAL
+#define FIELD_TYPE_NEWDECIMAL 246
+#endif
+#ifndef FIELD_TYPE_GEOMETRY
+#define FIELD_TYPE_GEOMETRY 255
+#endif
 
 /*
  * Globals
@@ -101,6 +111,8 @@
 
 struct program *mysql_result_program = NULL;
 
+static struct svalue mpq_program = SVALUE_INIT_FREE;
+
 /*
  * Functions
  */
@@ -151,7 +163,7 @@ void mysqlmod_parse_field(MYSQL_FIELD *field, int support_default)
     case FIELD_TYPE_DECIMAL:
       push_text("decimal");
       break;
-    case FIELD_TYPE_CHAR:
+    case FIELD_TYPE_CHAR:	/* Same as FIELD_TYPE_TINY. */
       push_text("char");
       break;
     case FIELD_TYPE_SHORT:
@@ -193,6 +205,12 @@ void mysqlmod_parse_field(MYSQL_FIELD *field, int support_default)
     case FIELD_TYPE_NEWDATE:
       push_text("newdate");
       break;
+    case FIELD_TYPE_BIT:
+      push_text("bit");
+      break;
+    case FIELD_TYPE_NEWDECIMAL:
+      push_text ("newdecimal");
+      break;
     case FIELD_TYPE_ENUM:
       push_text("enum");
       break;
@@ -217,11 +235,9 @@ void mysqlmod_parse_field(MYSQL_FIELD *field, int support_default)
     case FIELD_TYPE_STRING:
       push_text("string");
       break;
-#ifdef HAVE_FIELD_TYPE_GEOMETRY
     case FIELD_TYPE_GEOMETRY:
       push_text("geometry");
       break;
-#endif
     default:
       push_text("unknown");
       break;
@@ -305,6 +321,18 @@ static void f_create(INT32 args)
     Pike_error("Bad argument 1 to mysql_result()\n");
   }
 
+#ifdef OLD_SQL_COMPAT
+  PIKE_MYSQL_RES->typed_mode = 1;
+#else
+  PIKE_MYSQL_RES->typed_mode = 0;
+#endif
+  if (args > 1) {
+    if (sp[1-args].type != T_INT) {
+      Pike_error("Bad argument 2 to mysql_result()\n");
+    }
+    PIKE_MYSQL_RES->typed_mode = !!sp[1-args].u.integer;
+  }
+
   if (PIKE_MYSQL_RES->result) {
     mysql_free_result(PIKE_MYSQL_RES->result);
   }
@@ -562,26 +590,105 @@ static void f_fetch_row(INT32 args)
       if (row[i]) {
 	MYSQL_FIELD *field;
 
-	if ((field = mysql_fetch_field(PIKE_MYSQL_RES->result))) {
+	if (PIKE_MYSQL_RES->typed_mode &&
+	    (field = mysql_fetch_field(PIKE_MYSQL_RES->result))) {
 	  switch (field->type) {
-#ifdef OLD_SQL_COMPAT
 	    /* Integer types */
+          case FIELD_TYPE_LONGLONG:
+#ifdef AUTO_BIGNUM
+	    if (
+#ifdef HAVE_MYSQL_FETCH_LENGTHS
+		row_lengths[i]
+#else
+		strlen(row[i])
+#endif
+		>= 10) {
+	      push_text(row[i]);
+	      convert_stack_top_string_to_inumber(10);
+	      break;
+	    }
+#endif
+	    /* FALL_THROUGH */
+	  case FIELD_TYPE_TINY:
 	  case FIELD_TYPE_SHORT:
 	  case FIELD_TYPE_LONG:
 	  case FIELD_TYPE_INT24:
-	  case FIELD_TYPE_DECIMAL:
-#if 0
-	    /* This one will not always fit in an INT32 */
-          case FIELD_TYPE_LONGLONG:
-#endif /* 0 */
-	    push_int(atoi(row[i]));
+	    push_int(STRTOL(row[i], 0, 10));
 	    break;
+
+#if defined (HAVE_MYSQL_FETCH_LENGTHS) && defined (AUTO_BIGNUM)
+	  case FIELD_TYPE_BIT:
+	    if (row_lengths[i] <= SIZEOF_LONGEST) {
+	      unsigned LONGEST val = 0;
+	      unsigned j;
+	      for (j = 0; j < row_lengths[i]; j++)
+		val = (val << 8) | (unsigned char) row[i][j];
+	      push_ulongest (val);
+	    }
+	    else {
+	      push_string (make_shared_binary_string (row[i], row_lengths[i]));
+	      push_int (256);
+	      convert_stack_top_with_base_to_bignum();
+	      reduce_stack_top_bignum();
+	    }
+	    break;
+#endif
+
 	    /* Floating point types */
 	  case FIELD_TYPE_FLOAT:
 	  case FIELD_TYPE_DOUBLE:
 	    push_float(atof(row[i]));
 	    break;
-#endif /* OLD_SQL_COMPAT */
+
+	  case FIELD_TYPE_DECIMAL:
+	  case FIELD_TYPE_NEWDECIMAL:
+	    if (!field->decimals) {
+#ifdef AUTO_BIGNUM
+	      if (
+#ifdef HAVE_MYSQL_FETCH_LENGTHS
+		row_lengths[i]
+#else
+		strlen(row[i])
+#endif
+		>= 10
+	      ) {
+#ifdef HAVE_MYSQL_FETCH_LENGTHS
+		push_string(make_shared_binary_string(row[i], row_lengths[i]));
+#else
+		push_text(row[i]);
+#endif
+		convert_stack_top_string_to_inumber(10);
+		break;
+	      }
+#endif
+	      push_int(STRTOL(row[i], 0, 10));
+	      break;
+	    }
+
+	    /* Fixed-point number with fraction part. Make an mpq. */
+
+	    if (mpq_program.type == PIKE_T_FREE) {
+	      push_text ("Gmp.mpq");
+	      SAFE_APPLY_MASTER ("resolv", 1);
+	      if (Pike_sp[-1].type == T_PROGRAM)
+		move_svalue (&mpq_program, --Pike_sp);
+	      else {
+		pop_stack();
+		mpq_program.type = T_INT;
+	      }
+	    }
+
+	    if (mpq_program.type == T_PROGRAM) {
+#ifdef HAVE_MYSQL_FETCH_LENGTHS
+	      push_string(make_shared_binary_string(row[i], row_lengths[i]));
+#else
+	      push_text(row[i]);
+#endif
+	      apply_svalue (&mpq_program, 1);
+	      break;
+	    }
+	    /* FALL_THROUGH */
+
 	  default:
 #ifdef HAVE_MYSQL_FETCH_LENGTHS
 	    push_string(make_shared_binary_string(row[i], row_lengths[i]));
@@ -591,7 +698,7 @@ static void f_fetch_row(INT32 args)
 	    break;
 	  }
 	} else {
-	  /* Probably doesn't happen, but... */
+	  /* Everything is strings mode. */
 #ifdef HAVE_MYSQL_FETCH_LENGTHS
 	  push_string(make_shared_binary_string(row[i], row_lengths[i]));
 #else
@@ -599,8 +706,12 @@ static void f_fetch_row(INT32 args)
 #endif /* HAVE_MYSQL_FETCH_LENGTHS */
 	}
       } else {
-	/* NULL? */
-	push_undefined();
+	/* NULL */
+	if (PIKE_MYSQL_RES->typed_mode) {
+	  push_object(get_val_null());
+	} else {
+	  push_undefined();
+	}
 	if(i+1<num_fields)
 	  mysql_field_seek(PIKE_MYSQL_RES->result, i+1);
       }
@@ -679,6 +790,7 @@ static void f_fetch_json_result(INT32 args)
   FETCH_LENGTHS_TYPE *row_lengths;
 #endif /* HAVE_MYSQL_FETCH_LENGTHS */
   struct string_builder res;
+  ONERROR uwp;
   int r = 0;
 
   if (!PIKE_MYSQL_RES->result) {
@@ -686,6 +798,7 @@ static void f_fetch_json_result(INT32 args)
   }
 
   init_string_builder(&res, 0);
+  SET_ONERROR(uwp, free_string_builder, &res);
   string_builder_putchar(&res, '[');
 
   num_fields = mysql_num_fields(PIKE_MYSQL_RES->result);
@@ -731,6 +844,7 @@ next_row:
     /* No rows left in result */
     PIKE_MYSQL_RES->eof = 1;
     string_builder_putchar(&res, ']');
+    UNSET_ONERROR(uwp);
     push_string(finish_string_builder(&res));
   }
 
@@ -791,6 +905,7 @@ void exit_mysql_res(void)
     free_program(mysql_result_program);
     mysql_result_program = NULL;
   }
+  free_svalue (&mpq_program);
 }
 
 #else
