@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: odbc_result.c,v 1.64 2009/05/11 11:36:13 grubba Exp $
+|| $Id: odbc_result.c,v 1.67 2010/08/26 09:47:35 grubba Exp $
 */
 
 /*
@@ -238,7 +238,10 @@ static void odbc_fix_fields(void)
     odbc_field_sizes[i] = precision;
     switch(sql_type) {
     case SQL_CHAR:
-      push_text("char");
+#ifdef SQL_WCHAR
+    case SQL_WCHAR:
+#endif
+      push_text("string");
       break;
     case SQL_NUMERIC:
       push_text("numeric");
@@ -269,6 +272,9 @@ static void odbc_fix_fields(void)
       odbc_field_types[i] = SQL_C_CHAR;
       break;
     case SQL_VARCHAR:
+#ifdef SQL_WVARCHAR
+    case SQL_WVARCHAR:
+#endif
       push_text("var string");
       odbc_field_sizes[i] = 0;	/* Variable length */
       break;
@@ -279,6 +285,9 @@ static void odbc_fix_fields(void)
       push_text("time");
       break;
     case SQL_LONGVARCHAR:
+#ifdef SQL_WLONGVARCHAR
+    case SQL_WLONGVARCHAR:
+#endif
       push_text("var string");
       odbc_field_sizes[i] = 0;	/* Variable length */
       break;
@@ -756,6 +765,14 @@ static void f_fetch_row(INT32 args)
 	      free_string(s);
 	      push_empty_string();
 	      break;
+	    } else if (len == SQL_NULL_DATA) {
+	      free_string(s);
+	      if (num_strings > 1) {
+		num_strings--;
+	      } else {
+		push_undefined();
+	      }
+	      break;
 #ifdef SQL_NO_TOTAL
 	    } else if (len == SQL_NO_TOTAL) {
 	      /* More data remaining... */
