@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: oracle.c,v 1.97 2009/05/19 17:54:18 grubba Exp $
+|| $Id: 6a791690c07f7a26c729668fbd6f472ee32d1f4e $
 */
 
 /*
@@ -28,6 +28,7 @@
 #include "array.h"
 #include "stralloc.h"
 #include "interpret.h"
+#include "operators.h"
 #include "pike_types.h"
 #include "pike_memory.h"
 #include "threads.h"
@@ -1199,7 +1200,7 @@ static void f_fetch_fields(INT32 args)
 	case SQLT_VCS: /* varchar */
 	case SQLT_LNG: /* long */
 	case SQLT_LVC: /* long varchar */
-	  type_name="char";
+	  type_name="string";
 
 	  if (size > 0) {
 	    type = SQLT_CHR;
@@ -1630,6 +1631,16 @@ static void free_inout(struct inout *i)
 }
 
 
+static void f_eof(INT32 args)
+{
+  struct dbquery *dbquery = THIS_RESULT_QUERY;
+  pop_n_elems(args);
+  /* Note: Valid OCI statement types start at 1.
+   *       We use 0 to indicate EOF.
+   */
+  push_int(!dbquery->query_type);
+}
+
 static void f_fetch_row(INT32 args)
 {
   int i = 0;
@@ -1644,6 +1655,15 @@ static void f_fetch_row(INT32 args)
 
   pop_n_elems(args);
 
+  if (!dbquery->query_type) {
+    /* EOF already reached. */
+#ifdef ORACLE_DEBUG
+    fprintf(stderr," EOF already reached.\n", rc);
+#endif
+    push_undefined();
+    return;
+  }
+
   if(!dbquery->field_info)
   {
     f_fetch_fields(0);
@@ -1681,6 +1701,12 @@ static void f_fetch_row(INT32 args)
 
     if(rc==OCI_NO_DATA)
     {
+      /* No more rows int the result.
+       * NB: Oracle will complain with:
+       *       ORA-01002: fetch out of sequence
+       *     if any further attempts to fetch rows are performed.
+       */
+      dbquery->query_type = 0;
       push_undefined();
       return;
     }
@@ -2228,7 +2254,13 @@ static void f_big_typed_query_create(INT32 args)
 	  case T_STRING:
 	    addr = (ub1 *)value->u.string->str;
 	    len = value->u.string->len;
-	    fty = SQLT_LNG;
+	    if (len < 4000)
+            {
+	      rlen = 4000;
+	      fty = SQLT_CHR;
+	    }
+	    else
+	      fty = SQLT_LNG;
 	    break;
 	    
 	  case T_FLOAT:
@@ -2492,8 +2524,15 @@ static void dbdate_sprintf(INT32 args)
 		   &bsize,
 		   buffer);
 		
-  if(!IS_SUCCESS(rc))
+  if(!IS_SUCCESS(rc)) {
+    if (mode == 'O') {
+      /* Be fault tolerant in debug mode. */
+      pop_n_elems (args);
+      push_undefined();
+      return;
+    }
     ora_error_handler(get_global_error_handle(), rc,"OCIDateToText");
+  }
 
   pop_n_elems(args);
   push_text(buffer);
@@ -2563,10 +2602,32 @@ static void dbnull_sprintf(INT32 args)
 
 }
 
-static void dbnull_not(INT32 args)
+static void dbnull_eq(INT32 args)
 {
-  pop_n_elems(args);
-  push_int(1);
+  if(args<1) Pike_error("Too few arguments to Oracle.NULL->`==\n");
+  if(args > 1) pop_n_elems(args-1);
+  args = 1;
+  if (Pike_sp[-1].type != T_OBJECT) {
+    push_int(0);
+    return;
+  }
+  /* Check if it's an Oracle-NULL. */
+  stack_dup();
+  push_constant_text("is_oracle_null");
+  o_index();
+  if ((Pike_sp[-1].type != T_INT) || !Pike_sp[-1].u.integer) {
+    /* No - is it the generic Sql.NULL? */
+    pop_stack();
+    push_constant_text("is_sql_null");
+    o_index();
+    return;
+  }
+  pop_stack();
+  /* Yes - are the types the same? */
+  push_constant_text("type");
+  o_index();
+  push_svalue(&THIS_DBNULL->type);
+  f_eq(2);
 }
 
 PIKE_MODULE_INIT
@@ -2643,6 +2704,9 @@ PIKE_MODULE_INIT
 		     f_fetch_fields,tFunc(tNone,tArr(tMap(tStr,tMix))),
 		     ID_PUBLIC);
 	
+	/* function(:int) */
+	ADD_FUNCTION("eof", f_eof,tFunc(tNone,tInt), ID_PUBLIC);
+	
 	/* function(:int|array(string|int)) */
 	ADD_FUNCTION("fetch_row",
 		     f_fetch_row,tFunc(tNone,tOr(tInt,tArr(tOr(tStr,tInt)))),
@@ -2664,9 +2728,9 @@ PIKE_MODULE_INIT
 	map_variable("decimals","int",0,offset+OFFSETOF(dbresultinfo, decimals), T_INT);
 	
 	ADD_FUNCTION("`->=",protect_dbresultinfo,
-		     tFunc(tStr tMix,tVoid),0);
+		     tFunc(tStr tMix,tVoid), ID_STATIC);
 	ADD_FUNCTION("`[]=",protect_dbresultinfo,
-		     tFunc(tStr tMix,tVoid),0);
+		     tFunc(tStr tMix,tVoid), ID_STATIC);
 #ifdef ORACLE_DEBUG
 	set_gc_check_callback(gc_dbresultinfo_struct);
 #endif
@@ -2697,9 +2761,14 @@ PIKE_MODULE_INIT
   MY_END_CLASS(Date);
 
   MY_START_CLASS(dbnull); {
-    ADD_FUNCTION("create",dbnull_create,tFunc(tOr(tStr,tInt),tVoid),0);
-    ADD_FUNCTION("_sprintf",dbnull_sprintf,tFunc(tInt, tStr),0);
-    ADD_FUNCTION("`!",dbnull_not,tFunc(tVoid, tInt),0);
+    struct pike_string *null_string = make_shared_string("Null");
+    low_inherit(get_sql_null_prog(), NULL, -1, 0, 0, null_string);
+    free_string(null_string);
+    add_integer_constant("is_oracle_null", 1, 0);
+    ADD_FUNCTION("create", dbnull_create, tFunc(tOr(tStr, tInt), tVoid),
+		 ID_PROTECTED);
+    ADD_FUNCTION("_sprintf", dbnull_sprintf, tFunc(tInt, tStr), ID_PROTECTED);
+    ADD_FUNCTION("`==", dbnull_eq, tFunc(tMix, tInt01), ID_PROTECTED);
     map_variable("type","mixed",0,offset+OFFSETOF(dbnull, type), T_MIXED);
   }
   NULL_program=end_program();
