@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: efuns.c,v 1.189 2009/08/07 13:06:37 grubba Exp $
+|| $Id: 0743fe4b8692a813357929e59a33116a89dbb2a3 $
 */
 
 #include "global.h"
@@ -442,8 +442,8 @@ void f_file_stat(INT32 args)
 
   if(args<1)
     SIMPLE_TOO_FEW_ARGS_ERROR("file_stat", 1);
-  if(sp[-args].type != T_STRING)
-    SIMPLE_BAD_ARG_ERROR("file_stat", 1, "string");
+  if((sp[-args].type != T_STRING) || sp[-args].u.string->size_shift)
+    SIMPLE_BAD_ARG_ERROR("file_stat", 1, "string(0..255)");
 
   str = sp[-args].u.string;
   l = (args>1 && !UNSAFE_IS_ZERO(sp+1-args))?1:0;
@@ -1192,7 +1192,7 @@ void f_get_dir(INT32 args)
   }
 
 #ifdef READDIR_DEBUG
-  fprintf(stderr, "FindFirstFile(\"%S\")...\n", STR1(str));
+  fprintf(stderr, "FindFirstFile(\"%S\")...\n", STR1(sb.s));
 #endif /* READDIR_DEBUG */
 
   dir = FindFirstFileW(STR1(sb.s), &d);
@@ -1200,40 +1200,66 @@ void f_get_dir(INT32 args)
   free_string_builder(&sb);
 
   if (dir == DO_NOT_WARN(INVALID_HANDLE_VALUE)) {
+    int err = GetLastError();
 #ifdef READDIR_DEBUG
-    fprintf(stderr, "  INVALID_HANDLE_VALUE\n");
+    fprintf(stderr, "  INVALID_HANDLE_VALUE, error %d\n", err);
 #endif /* READDIR_DEBUG */
 
     pop_n_elems(args);
-    push_int(0);
+    if (err == ERROR_FILE_NOT_FOUND) {
+      /* Normally there should at least be a "." entry, so this seldom
+       * happens. But it seems unwise to count on it, considering this
+       * being Windows and all..
+       *
+       * Note: The error is ERROR_PATH_NOT_FOUND if the directory
+       * doesn't exist.
+       */
+      push_empty_array();
+    }
+    else {
+      set_errno_from_win32_error (err);
+      push_int(0);
+    }
     return;
   }
 
-  BEGIN_AGGREGATE_ARRAY(10);
+  {
+    int err;
 
-  do {
+    BEGIN_AGGREGATE_ARRAY(10);
+
+    do {
 #ifdef READDIR_DEBUG
-    fprintf(stderr, "  \"%S\"\n", d.cFileName);
+      fprintf(stderr, "  \"%S\"\n", d.cFileName);
 #endif /* READDIR_DEBUG */
-    /* Filter "." and ".." from the list. */
-    if(d.cFileName[0]=='.')
-    {
-      if(!d.cFileName[1]) continue;
-      if(d.cFileName[1]=='.' && !d.cFileName[2]) continue;
+      /* Filter "." and ".." from the list. */
+      if(d.cFileName[0]=='.')
+      {
+	if(!d.cFileName[1]) continue;
+	if(d.cFileName[1]=='.' && !d.cFileName[2]) continue;
+      }
+      push_string(make_shared_binary_string1(d.cFileName, wcslen(d.cFileName)));
+      DO_AGGREGATE_ARRAY(120);
+    } while(FindNextFileW(dir, &d));
+    err = GetLastError();
+
+#ifdef READDIR_DEBUG
+    fprintf(stderr, "  DONE, error %d\n", err);
+#endif /* READDIR_DEBUG */
+
+    FindClose(dir);
+
+    END_AGGREGATE_ARRAY;
+
+    stack_pop_n_elems_keep_top(args);
+
+    if (err != ERROR_SUCCESS && err != ERROR_NO_MORE_FILES) {
+      set_errno_from_win32_error (err);
+      pop_stack();
+      push_int (0);
+      return;
     }
-    push_string(make_shared_binary_string1(d.cFileName, wcslen(d.cFileName)));
-    DO_AGGREGATE_ARRAY(120);
-  } while(FindNextFileW(dir, &d));
-
-#ifdef READDIR_DEBUG
-  fprintf(stderr, "  DONE\n");
-#endif /* READDIR_DEBUG */
-
-  FindClose(dir);
-
-  END_AGGREGATE_ARRAY;
-
-  stack_pop_n_elems_keep_top(args);
+  }
 
 #else /* !__NT__ */
 
@@ -1601,7 +1627,7 @@ void f_exece(INT32 args)
 
   if(en)
   {
-    INT32 e;
+    INT32 e, i = 0;
     struct keypair *k;
 
     env=(char **)malloc((1+m_sizeof(en)) * sizeof(char *));
@@ -1615,12 +1641,11 @@ void f_exece(INT32 args)
       push_constant_text("=");
       push_string(k->val.u.string);
       f_add(3);
-      env[e]=sp[-1].u.string->str;
-      sp--;
-      dmalloc_touch_svalue(sp);
+      env[i++]=sp[-1].u.string->str;
+      dmalloc_touch_svalue(sp-1);
     }
 
-    env[e]=0;
+    env[i]=0;
   }else{
     env=environ;
   }
