@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: system.c,v 1.187 2009/05/28 11:54:37 grubba Exp $
+|| $Id: fb5d65cd5b3269e95715ad6d0b5ce4d8816f671b $
 */
 
 /*
@@ -110,6 +110,10 @@
 #include <netinfo/ni.h>
 #endif
 
+#ifdef HAVE_SYS_LOADAVG_H
+#include <sys/loadavg.h>
+#endif
+
 #include "dmalloc.h"
 
 
@@ -1856,6 +1860,23 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port, i
     /* Avoid creating an IPv6 address for "*". */
     /* For IN6ADDR_ANY, use "::". */
     hints.ai_family = PF_INET;
+#if 0
+    /* We want to be able to run the compiled Pike
+     * even on systems without IPv6...
+     */
+#if defined(PF_INET6) && defined(AI_V4MAPPED)
+  } else {
+    /* Map all addresses to the IPv6 namespace,
+     * to avoid address conflicts when once end
+     * of the socket is IPv4 and one end is IPv6.
+     * This is needed on eg Linux where a socket
+     * bound to ::FFFF:127.0.0.1 can't connect
+     * to the IPv4 address 127.0.0.1.
+     */
+    hints.ai_family = PF_INET6;
+    hints.ai_flags = AI_V4MAPPED;
+#endif
+#endif /* 0 */
   }
   hints.ai_protocol = (udp? IPPROTO_UDP:IPPROTO_TCP);
   if(!service)
@@ -2551,7 +2572,7 @@ static void f_setrlimit(INT32 args)
        sp[1-args].u.integer<-1) 
       SIMPLE_BAD_ARG_ERROR("setrlimit",2,"int(-1..)");
    if (sp[2-args].type!=T_INT ||
-       sp[1-args].u.integer<-1) 
+       sp[2-args].u.integer<-1) 
       SIMPLE_BAD_ARG_ERROR("setrlimit",3,"int(-1..)");
 
 #ifdef RLIM_INFINITY
@@ -2844,6 +2865,38 @@ static void f_get_netinfo_property(INT32 args)
 }
 #endif
 
+#ifdef HAVE_GETLOADAVG
+/*! @decl array(float) getloadavg()
+ *! Get system load averages.
+ *!
+ *! @returns
+ *! @array
+ *!   @elem float 0
+ *!     Load average over the last minute.
+ *!   @elem float 1
+ *!     Load average over the last 5 minutes.
+ *!   @elem float 2
+ *!     Load average over the last 15 minutes.
+ *! @endarray
+ */
+void f_system_getloadavg(INT32 args)
+{
+  double load[3] = { 0.0, 0.0, 0.0 };
+  int i;
+
+  pop_n_elems(args); // No args.
+
+  /* May return less than requested number of values */
+  if (getloadavg(load, 3) == -1) {
+    Pike_error("getloadavg failed.\n");
+  }
+  
+  for (i = 0; i <= 2; i++) {
+    push_float((FLOAT_TYPE)load[i]);
+  }
+  f_aggregate(3);
+}
+#endif // HAVE_GETLOADAVG
 
 #ifdef HAVE_RDTSC
 
@@ -3451,6 +3504,10 @@ PIKE_MODULE_INIT
 	       tFunc(tStr tStr tStr, tArray), 0);
 #endif /* NETINFO */
 
+#ifdef HAVE_GETLOADAVG
+  ADD_FUNCTION("getloadavg", f_system_getloadavg, tFunc(tNone,tArr(tFloat)), 0);
+#endif
+
   init_passwd();
   init_system_memory();
 
