@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: pike_memory.h,v 1.58 2008/06/23 22:21:02 mast Exp $
+|| $Id: pike_memory.h,v 1.59 2010/09/21 15:24:23 grubba Exp $
 */
 
 #ifndef MEMORY_H
@@ -200,7 +200,7 @@ void exit_pike_memory (void);
     size_t ret;						\
   							\
     ret = 9248339*len;					\
-    if(len<mlen)					\
+    if(len<=mlen)					\
       mlen=len;						\
     else						\
     {							\
