@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: port.c,v 1.87 2008/01/19 14:51:53 grubba Exp $
+|| $Id: port.c,v 1.89 2010/02/18 14:50:41 srb Exp $
 */
 
 /*
@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <float.h>
 #include <string.h>
+#include <time.h>
 
 #ifndef HAVE_ISSPACE
 PMOD_EXPORT const char Pike_isspace_vector[] =
@@ -155,6 +156,33 @@ PMOD_EXPORT unsigned INT32 my_rand(void)
   return rndbuf[rnd_index] += rndbuf[rnd_index+RNDJUMP-(rnd_index<RNDBUF-RNDJUMP?0:RNDBUF)];
 }
 
+PMOD_EXPORT void sysleep(double left)
+{
+#ifdef __NT__
+  Sleep(DO_NOT_WARN((int)(left*1000+0.5)));
+#elif defined(HAVE_NANOSLEEP)
+  {
+    struct timespec req;
+    left+=5e-10;
+    req.tv_nsec=(left - (req.tv_sec=left))*1e9;
+    nanosleep(&req,(void*)0);
+  }
+#elif defined(HAVE_POLL)
+  {
+    /* MacOS X is stupid, and requires a non-NULL pollfd pointer. */
+    struct pollfd sentinel;
+    poll(&sentinel, 0, (int)(left*1000+0.5));
+  }
+#else
+  {
+    struct timeval t3;
+    left+=5e-7;
+    t3.tv_usec=(left - (t3.tv_sec=left))*1e6;
+    select(0,0,0,0,&t3);
+  }
+#endif
+}
+
 #ifndef CONFIGURE_TEST
 
 PMOD_EXPORT /*@null@*/ void *pike_realloc(void *ptr, size_t sz)
@@ -175,7 +203,7 @@ PMOD_EXPORT /*@null@*/ void *pike_realloc(void *ptr, size_t sz)
 			islower(x) ? (x) + 10 - 'a' : (x) + 10 - 'A')
 #define MBASE	('z' - 'a' + 1 + 10)
 
-long STRTOL(const char *str, char **ptr, int base)
+PMOD_EXPORT long STRTOL(const char *str, char **ptr, int base)
 {
   /* Note: Code duplication in STRTOL_PCHARP and pcharp_to_svalue_inumber. */
 
