@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: port.h,v 1.66 2009/05/28 13:10:48 grubba Exp $
+|| $Id: port.h,v 1.68 2010/02/18 14:50:41 srb Exp $
 */
 
 #ifndef PORT_H
@@ -158,7 +158,7 @@ time_t TIME(time_t *);
 #define RINT rint
 #define RINTL rintl
 
-long STRTOL(const char *str,char **ptr,int base);
+PMOD_EXPORT long STRTOL(const char *str,char **ptr,int base);
 PMOD_EXPORT double STRTOD(const char * nptr, char **endptr);
 
 #ifndef HAVE_STRCSPN
@@ -317,6 +317,8 @@ PMOD_EXPORT INT32 EXTRACT_INT_(unsigned char *p);
 PMOD_EXPORT unsigned INT32 my_rand(void);
 PMOD_EXPORT void my_srand(INT32 seed);
 
+PMOD_EXPORT void sysleep(double left);
+
 PMOD_EXPORT /*@null@*/ void *pike_realloc(void *ptr, size_t sz);
 
 #ifdef OWN_GETHRTIME
