@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: support.c,v 1.20 2008/01/30 13:29:55 per Exp $
+|| $Id: support.c,v 1.28 2010/09/19 20:12:25 marcus Exp $
 */
 
 #include <version.h>
@@ -86,7 +86,8 @@ int get_color_from_pikecolor(struct object *o, INT_TYPE *r, INT_TYPE *g, INT_TYP
   return 1;
 }
 
-GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GdkImage *i) {
+GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GObject **pi) {
+  GdkImage *i;
   GdkColormap *col=gdk_colormap_get_system();
   GdkVisual *vis=gdk_visual_get_system();
   struct image *img_data;
@@ -103,16 +104,20 @@ GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GdkImage *i) {
 
   if (x==0 || y==0)
     Pike_error("Size of image must be > 0x0\n");
-  if (i) {
-    if ((i->width!=x) || (i->height!=y)) {
-      gdk_image_destroy((void *)i);
+  if (pi) {
+    i = GDK_IMAGE(*pi);
+    if (i != NULL && ((i->width!=x) || (i->height!=y))) {
+      g_object_unref(i);
       i=NULL;
     }
-  }
+  } else
+    i=NULL;
   if (!i) {
     PFTIME("Create");
     i=(void *)gdk_image_new(fast,vis,x,y);
   }
+  if (pi)
+    *pi = G_OBJECT(i);
 
   if (!i)
     Pike_error("Failed to create gdkimage\n");
@@ -767,17 +772,11 @@ void pgtk2_signal_func_wrapper(struct signal_data *d,
   for (i=0; i<n_params; i++) {
     pgtk2_push_gvalue_rt(&(param_values[i]));
   }
-  if (n_params)
-    f_aggregate(n_params);
   push_svalue(&d->args);
-  if (n_params)
-    apply_svalue(&d->cb,3);
-  else
-    apply_svalue(&d->cb,2);
-  if (return_value) {
-    pgtk2_set_value(return_value,&Pike_sp[-1]);
-    pop_stack();
-  }
+  apply_svalue(&d->cb,2+n_params);
+  if (return_value && G_VALUE_TYPE(return_value) != 0 )
+      pgtk2_set_value(return_value,&Pike_sp[-1]);
+  pop_stack();
 }
 
 void pgtk2_free_signal_data(struct signal_data *s, GClosure *gcl) {
@@ -787,9 +786,12 @@ void pgtk2_free_signal_data(struct signal_data *s, GClosure *gcl) {
 }
 
 void pgtk2_push_gchar(const gchar *s) {
-  push_text(s); 
-  push_int(1);
-  f_utf8_to_string(2);
+  if (s) {
+    push_text(s); 
+    push_int(1);
+    f_utf8_to_string(2);
+  } else
+    push_int(0);
 }
 
 gchar *pgtk2_get_str(struct svalue *sv) {
@@ -814,6 +816,20 @@ void pgtk2_free_str(gchar *s) {
   g_free(s);
 }
 
+void pgtk2_get_string_arg_with_sprintf( INT32 args )
+{
+  if( args < 1 )
+    Pike_error("Too few arguments, %d required, got %d\n", 1, args);
+
+  if( Pike_sp[-args].type != PIKE_T_STRING )
+    Pike_error("Illegal argument %d, expected string\n", 0);
+
+  if( args > 1 )
+    f_sprintf(args);
+
+  f_string_to_utf8(1);
+}
+
 void pgtk2_default__sprintf(int args, int offset, int len) {
   int mode = 0;
   if (args>0 && Pike_sp[-args].type==PIKE_T_INT)
@@ -1245,8 +1261,10 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
       } else
 	g_value_set_pointer(gv,NULL);
       break;
+      case 0: // void
+          break;
     default:
-      Pike_error("Unable to handle type %d - %s.\n",gt,g_type_name(gt));
+        Pike_error("Unable to handle type %d - %s.\n",gt,g_type_name(gt) ?g_type_name(gt): "unnamed" );
   }
 }
 
@@ -1341,3 +1359,141 @@ int pgtk2_tree_view_row_separator_func(GtkTreeModel *model,
   return res;
 }
 
+
+int pgtk2_entry_completion_match_func( GtkEntryCompletion *x,
+                                       const gchar *key,
+                                       GtkTreeIter *iter,
+                                       struct signal_data *d)
+{
+  int res;
+  push_gobject(x);
+  pgtk2_push_gchar( key );
+  push_gobjectclass(iter,pgtk2_tree_iter_program);
+  apply_svalue( &d->cb, 3 );
+  res = Pike_sp[-1].u.integer;
+  pop_stack();
+  return res;
+}
+
+void add_property_docs(GType type, GString *str) {
+  GObjectClass *class;
+  GParamSpec **props;
+  guint n=0,i;
+  gboolean has_prop=FALSE;
+  G_CONST_RETURN gchar *blurb=NULL;
+
+  class=g_type_class_ref(type);
+  props=g_object_class_list_properties(class,&n);
+
+  for (i=0; i<n; i++) {
+    if (props[i]->owner_type!=type)
+      continue; /* these are from a parent type */
+
+    if (!has_prop) {
+      g_string_append_printf(str,"Properties from %s:\n",g_type_name(type));
+      has_prop=TRUE;
+    }
+    g_string_append_printf(str,"  %s - %s: %s\n",
+		g_param_spec_get_name(props[i]),
+		g_type_name(props[i]->value_type),
+		g_param_spec_get_nick(props[i]));
+    blurb=g_param_spec_get_blurb(props[i]);
+    if (blurb)
+      g_string_append_printf(str,"    %s\n",blurb);
+  }
+  g_free(props);
+  if (has_prop)
+    g_string_append(str,"\n");
+  g_type_class_unref(class);
+}
+
+void add_signal_docs(GType type, GString *str) {
+  GTypeClass *class=NULL;
+  guint *signal_ids,n=0,i;
+
+  if (G_TYPE_IS_CLASSED(type))
+    class=g_type_class_ref(type);
+  signal_ids=g_signal_list_ids(type,&n);
+
+  if (n>0) {
+    g_string_append_printf(str,"Signals from %s:\n",g_type_name(type));
+    for (i=0; i<n; i++) {
+      GSignalQuery q;
+      guint j;
+
+      g_signal_query(signal_ids[i],&q);
+      g_string_append(str,"  ");
+      g_string_append(str,q.signal_name);
+      g_string_append(str," (");
+      for (j=0; j<q.n_params; j++) {
+	g_string_append(str,g_type_name(q.param_types[j]));
+	if (j!=q.n_params-1)
+	  g_string_append(str,", ");
+      }
+      g_string_append(str,")");
+      if (q.return_type && q.return_type!=G_TYPE_NONE) {
+	g_string_append(str," -> ");
+	g_string_append(str,g_type_name(q.return_type));
+      }
+      g_string_append(str,"\n");
+    }
+    g_free(signal_ids);
+    g_string_append(str,"\n");
+  }
+  if (class)
+    g_type_class_unref(class);
+}
+
+struct svalue *pgtk2_get_doc(GObject *o, int pushv) {
+  GType type=0;
+  GString *str;
+  struct svalue *sv;
+
+/*
+  if (o)
+    type=G_OBJECT_TYPE(G_OBJECT(o)->obj);
+  else
+    return NULL;
+*/
+  type=G_OBJECT_TYPE(o);
+  str=g_string_new_len(NULL,512);
+
+  if (g_type_is_a(type,G_TYPE_INTERFACE))
+    g_string_append_printf(str,"Interface %s\n\n",g_type_name(type));
+  else if (g_type_is_a(type,G_TYPE_OBJECT))
+    g_string_append_printf(str,"Object %s\n\n",g_type_name(type));
+
+  if (g_type_is_a(type,G_TYPE_OBJECT)) {
+    GType parent=G_TYPE_OBJECT;
+    GArray *parents=g_array_new(FALSE,FALSE,sizeof(GType));
+    int ip;
+
+    while (parent) {
+      g_array_append_val(parents,parent);
+      parent=g_type_next_base(type,parent);
+    }
+    for (ip=parents->len-1; ip>=0; --ip) {
+      GType *interfaces;
+      guint n,i;
+
+      parent=g_array_index(parents,GType,ip);
+      add_signal_docs(parent,str);
+      add_property_docs(parent,str);
+
+      interfaces=g_type_interfaces(parent,&n);
+      for (i=0; i<n; i++)
+	add_signal_docs(interfaces[i],str);
+      g_free(interfaces);
+    }
+    g_array_free(parents,TRUE);
+  }
+  push_string(make_shared_binary_string(str->str,str->len));
+  g_string_free(str,TRUE);
+  if (!pushv) {
+    sv=g_new0(struct svalue,1);
+    assign_svalue_no_free(sv,&Pike_sp[-1]);
+    pop_stack();
+    return sv;
+  }
+  return sv;
+}
