@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: stralloc.c,v 1.236 2009/04/10 11:43:57 grubba Exp $
+|| $Id: stralloc.c,v 1.239 2010/06/15 16:10:19 grubba Exp $
 */
 
 #include "global.h"
@@ -2317,12 +2317,9 @@ PMOD_EXPORT void string_builder_putchar(struct string_builder *s, int ch)
   ptrdiff_t i;
   int mag = min_magnitude(ch);
 
-  if (mag > s->s->size_shift) {
-    string_build_mkspace(s, 1, mag);
+  string_build_mkspace(s, 1, mag);
+  if (mag > s->known_shift) {
     s->known_shift = mag;
-  } else if (((size_t)s->s->len) >= ((size_t)s->malloced)) {
-    string_build_mkspace(s, 1, mag);
-    s->known_shift = MAXIMUM(mag, s->known_shift);
   }
   i = s->s->len++;
   low_set_index(s->s,i,ch);
@@ -2330,6 +2327,40 @@ PMOD_EXPORT void string_builder_putchar(struct string_builder *s, int ch)
   s->s->str[s->s->len << s->s->size_shift] = 0;
 }
 
+PMOD_EXPORT void string_builder_putchars(struct string_builder *s, int ch,
+					 ptrdiff_t count)
+{
+  ptrdiff_t len = s->s->len;
+  int mag = min_magnitude(ch);
+
+  string_build_mkspace(s, count, mag);
+  if (mag > s->known_shift) {
+    s->known_shift = mag;
+  }
+
+  switch (s->s->size_shift) {
+    case 0:
+      MEMSET (STR0 (s->s) + s->s->len, ch, count);
+      break;
+    case 1: {
+      int i;
+      for (i = 0; i < count; i++)
+	(STR1 (s->s) + s->s->len)[i] = ch;
+      break;
+    }
+    case 2: {
+      int i;
+      for (i = 0; i < count; i++)
+	(STR2 (s->s) + s->s->len)[i] = ch;
+      break;
+    }
+  }
+
+  s->s->len += count;
+  /* Ensure NUL-termination */
+  s->s->str[s->s->len << s->s->size_shift] = 0;
+}
+
 
 PMOD_EXPORT void string_builder_binary_strcat0(struct string_builder *s,
 					       const p_wchar0 *str, ptrdiff_t len)
@@ -2945,13 +2976,17 @@ PMOD_EXPORT void string_builder_vsprintf(struct string_builder *s,
 					flags, min_width, precision);
 	  break;
 
-	  /* FIMXE: TODO: Doubles (ie 'a', 'e', 'E', 'g', 'G'). */
-
 	  /* %f used in modules/Image/colors.c. */
+	case 'a':
+	case 'e':
+	case 'E':
 	case 'f':
+	case 'g':
+	case 'G':
 	  {
 	    double val = va_arg(args, double);
 	    size_t bytes;
+	    char nfmt[] = { '%', fmt[-1], 0 };
 
 	    if (PIKE_ISNAN(val)) {
 	      /* NaN */
@@ -2964,23 +2999,20 @@ PMOD_EXPORT void string_builder_vsprintf(struct string_builder *s,
 	    } else if (flags & APPEND_SIGNED) {
 	      string_builder_putchar(s, '+');
 	    }
-	    if (val+val == val) {
-	      if (val > 0.0) {
-		/* Infinity */
-		string_builder_strcat(s, "inf");
-	      } else {
-		string_builder_strcat(s, "0.0");
-	      }
+	    if ((val+val == val) && (val > 0.0)) {
+	      /* Infinity */
+	      string_builder_strcat(s, "inf");
 	      break;
 	    }
 	    /* FIXME: Field lengths and precision. */
-	    if ((bytes = SNPRINTF(NULL, 0, "%f", val))) {
+	    if ((bytes = SNPRINTF(NULL, 0, nfmt, val))) {
 	      p_wchar0 *p = string_builder_allocate(s, bytes, 0);
-	      size_t check = SNPRINTF((char*)p, bytes+1, "%f", val);
+	      size_t check = SNPRINTF((char*)p, bytes+1, nfmt, val);
 	      if (check != bytes) {
-		Pike_fatal("string_builder_vsprintf(): snprintf(%f) is not "
-			   "trustworthy: %"PRINTSIZET"u != %"PRINTSIZET"u\n",
-			   val, bytes, check);
+		Pike_fatal("string_builder_vsprintf(): snprintf(\"%s\", %f) "
+			   "is not trustworthy: "
+			   "%"PRINTSIZET"u != %"PRINTSIZET"u\n",
+			   nfmt, val, bytes, check);
 	      }
 	      if (s->s->size_shift) {
 		/* We need to widen the string we just wrote. */
@@ -3002,7 +3034,7 @@ PMOD_EXPORT void string_builder_vsprintf(struct string_builder *s,
 
 	default:
 	  Pike_fatal("string_builder_vsprintf(): Invalid formatting method: "
-		     "'%c' 0x%x.\n", fmt[-1], fmt[-1]);
+		     "\"%%%c\" 0x%x.\n", (fmt[-1] & 0xff), fmt[-1]);
 	}
 	break;
       }
@@ -3094,7 +3126,8 @@ PMOD_EXPORT long STRTOL_PCHARP(PCHARP str, PCHARP *ptr, int base)
     {
     case '-':
       neg++;
-    case '+':			/* fall-through */
+      /* FALL_THROUGH */
+    case '+':
       INC_PCHARP(str,1);
       c=EXTRACT_PCHARP(str);
     }
