@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: threads.c,v 1.280 2009/04/21 16:02:29 jonasw Exp $
+|| $Id: 8daaa74edd44e69fb0f6d123d9ca1d060eda907d $
 */
 
 #include "global.h"
@@ -951,6 +951,15 @@ static void check_threads(struct callback *cb, void *arg, void * arg2)
 #endif
   if (clock() - thread_start_clock < (clock_t) (CLOCKS_PER_SEC / 20))
     return;
+  {
+    clock_t clock_now = clock();
+    if (clock_now - thread_start_clock < 0)
+      /* clock counter has wrapped since the start of the time slice.
+       * Let's reset and yield. */
+      thread_start_clock = 0;
+    else if (clock_now - thread_start_clock < (clock_t) (CLOCKS_PER_SEC / 20))
+      return;
+  }
 #else
   static int div_;
   if(div_++ & 255)
@@ -972,6 +981,11 @@ static void check_threads(struct callback *cb, void *arg, void * arg2)
   }
 #endif
 
+  pike_thread_yield();
+}
+
+PMOD_EXPORT void pike_thread_yield(void)
+{
   DEBUG_CHECK_THREAD();
 
   THREADS_ALLOW();
@@ -1095,6 +1109,14 @@ TH_RETURN_TYPE new_thread_func(void *data)
   {
     if(throw_severity <= THROW_ERROR)
       call_handle_error();
+    if(throw_severity == THROW_EXIT)
+    {
+      /* This is too early to get a clean exit if DO_PIKE_CLEANUP is
+       * active. Otoh it cannot be done later since it requires the
+       * evaluator stacks in the gc calls. It's difficult to solve
+       * without handing over the cleanup duty to the main thread. */
+      pike_do_exit(throw_value.u.integer);
+    }
   } else {
     INT32 args=arg.args->size;
     back.severity=THROW_EXIT;
@@ -1107,8 +1129,6 @@ TH_RETURN_TYPE new_thread_func(void *data)
     if (thread_state->thread_obj)
       assign_svalue(&thread_state->result, Pike_sp-1);
     pop_stack();
-
-    throw_severity = THROW_N_A;
   }
 
   UNSETJMP(back);
@@ -1162,11 +1182,6 @@ TH_RETURN_TYPE new_thread_func(void *data)
   debug_print_rusage (stderr);
 #endif
 
-  if (throw_severity == THROW_EXIT)
-    /* Do this after all thread cleanup to avoid false alarms if using
-     * DO_PIKE_CLEANUP. */
-    pike_do_exit (throw_value.u.integer);
-
   /* FIXME: What about threads_disable? */
   mt_unlock_interpreter();
   th_exit(0);
@@ -2666,13 +2681,7 @@ void cleanup_all_other_threads (void)
 
   while (num_pending_interrupts && time (NULL) < timeout) {
     THREADS_ALLOW();
-#ifdef __NT__
-    Sleep (1);
-#elif defined (HAVE_USLEEP)
-    usleep (1000);
-#else
-    sleep (1);
-#endif
+    sysleep(1.0);
     THREADS_DISALLOW();
   }
 
